% Extract a seam and edge on a welding image and display
%
% Jiansheng Chen, Guangda Su and Shoubin Xiang, 
% Robust Welding Seam Tracking using  Image Seam 
% Extraction, Science and Technology of Welding and 
% Joining, Vol. 17, No. 2, Feb. 2012.
%
%01-10-12, Jiansheng Chen

function edgeseam(I)

    I = im2double(I);
    try
        I = rgb2gray(I);
    catch exception
    end

    % extract directly the single seam first
    minpath = extractseam(exp(-I*3));
    
    % calculate the gradient of I
    [Ix,Iy] = gradient(I);
    Igrad = sqrt(Ix.^2+Iy.^2);
    
    % left side edge extraction
    mask = zeros(size(I));
    for i = 1:size(mask, 1)
        mask(i, minpath(i):end) = NaN;
    end
    minpath_left = extractseam(exp(-Igrad*5)+mask);

    % right side edge extraction
    mask = zeros(size(I));
    for i = 1:size(mask, 1)
        mask(i, 1:minpath(i)) = NaN;
    end
    minpath_right = extractseam(exp(-Igrad*5)+mask);
    
    minpath_mid = round((minpath_right+minpath_left)/2);
    
    img = zeros(size(I,1), size(I,2), 3);
    img(:,:,1) = I;
    img(:,:,2) = I;
    img(:,:,3) = I;
    for i = 1:length(minpath_left)
        img(i, minpath_left(i), 1) = 1;
        img(i, minpath_left(i), 2) = 0;
        img(i, minpath_left(i), 3) = 0;
    end
    for i = 1:length(minpath_right)
        img(i, minpath_right(i), 1) = 1;
        img(i, minpath_right(i), 2) = 0;
        img(i, minpath_right(i), 3) = 0;
    end    
 
	for i = 1:length(minpath_mid)
        img(i, minpath_mid(i), 1) = 1;
        img(i, minpath_mid(i), 2) = 0;
        img(i, minpath_mid(i), 3) = 0;
    end    
    
    figure('name', 'Edge seam extraction');
    imshow(img);
    
end