% Extract a seam given an engery map
%
% Jiansheng Chen, Guangda Su and Shoubin Xiang, 
% Robust Welding Seam Tracking using  Image Seam 
% Extraction, Science and Technology of Welding and 
% Joining, Vol. 17, No. 2, Feb. 2012.
%
%01-10-12, Jiansheng Chen

% input:
%       E: the energy map in double
% output:
%       minpath: index of the pixels on the extracted seam 

function minpath = extractseam(E)
    
    Emin = E;    
    alpha = 1.25; %penalty for diagonal edge
    Ealpha = (alpha-1)*E;
    for i = 2:size(Emin,1)
        rowmin = min([NaN NaN Emin(i-1,:)+Ealpha(i-1,:); ...
                             NaN Emin(i-1,:) NaN; ...
                             Emin(i-1, :)+Ealpha(i-1,:) NaN NaN]);
         Emin(i,:) = Emin(i,:)+rowmin(2:end-1);                
    end
    
    % find one of the minimal length path
    minpath = zeros(size(E, 1), 1);
    midx = find(Emin(end,:) == min(Emin(end,:)));
    minpath(end) = midx(ceil(rand*length(midx)));
    
    % find the minimum seam
    for i=size(E,1)-1:-1:1
        tmp = Emin(i,max(minpath(i+1)-1,1):min(minpath(i+1)+1,size(Emin,2)));
        [~,idx]=min(tmp);
        minpath(i)=idx+minpath(i+1)-1-(minpath(i+1)>1);
    end

end